

package gitzel.util;

import java.io.*;
import java.util.*;

/**
 * This class allows reading ini files from any input stream, file or http-doc.
 * It is basically a portation of the c++ class InitReader, I wrote for LS PI4.
 * Here is the original header file text:
   This class takes an input stream and extracts init-file infos.
   These are stored in a map and are accesible via methods.
   The input stream (usually a file) is parsed like this:
   HEADERNAME: DATA \n
   In order to parse other formats simply create a subclass to
   this one and overwrite the protected member 'parse' with
   your own function that suits your definition of an init file
   line.
 * @author Ralf Gitzel
 * @version 1.0
 */
public class IniReader
{
/** The input stream we use... */
BufferedReader IStream;

/** The Map to store all entries. */
private Map Data = new HashMap();

/** Pass an input stream to the ini reader to have it know where to look... */
public IniReader(InputStream in) throws IOException
  {
  this(new BufferedReader(new InputStreamReader(in)));
  }

/** The IniReader takes the given String as input stream. */
public IniReader(String data) throws IOException
  {
  this(new BufferedReader(new StringReader(data)));
  }

public IniReader(BufferedReader r) throws IOException
  {
  String line;
  do{
      line = r.readLine();
      parse(line); // call the parse method
    } while (line != null);
  }

/** gives the data correpsonding to the given entry. If entry does not exist,
 *  an empty string is returned. */
public String getEntry(String field)
  {
  return (String) Data.get(field);
  }

/** alter header X information (use save function to update on disk as
	well). Function returns true if the header did not exist yet (this
   indicates that a new entry was created). */
public boolean setEntry(String X, String data)
  {
  Object o = Data.put(X, data);
  if (o == null) return true;
  return false;
  }

/** delete the entry X */
public void deleteEntry(String X)
  {
  Data.remove(X);
  }

/** save the header info to the given output stream. A false indicates
    failure */
public void save(OutputStream o)
  {
  PrintWriter out = new PrintWriter(o);
  Set keyz = Data.keySet();
  Iterator it = keyz.iterator();
  while (it.hasNext())
    {
    String k = (String) it.next();
    String nxt = (String) Data.get(k);
    out.println(k+":\t"+nxt);
    }
  out.close();
  }

/** a shortcut function that allows giving a file name instead
    of a stream for saving */
public void save(String filename) throws FileNotFoundException
  {
  save(new FileOutputStream(filename));
  }

/** parses a line from the input file. This function is 'virtual' to
    allow easy creation of other file formats via subclasses and overwriting
    this method. */
protected void parse(String line)
  {
  if (line == null) return;         // ignore eof-lines
  if (line.equals("")) return;      // ignore empty lines
  if (line.startsWith("#")) return; // ignore comment lines
  String hdr, data;
  // get position of first ':'
  int cut = line.indexOf(':');
  // get substring before :
  hdr = line.substring(0, cut);
  // get substring after and trim off whitespaces
  data = line.substring(cut+1);
  data = data.trim();
  // is this an empty line?
  if (hdr == new String("")) return;

  // insert new entry into Data, if same hdr as before, add to old header separated by \n
  String temp = getEntry(hdr);
  if(temp != null)
    {
    String d = temp + "\n" + data;
    setEntry(hdr, d);
    }
  else setEntry(hdr, data);
  }
}