
//Titel:Viror Animationsbibliothek
//Version:
//Copyright:Copyright (c) 1998
//Autor:Ralf Gitzel
//Organisation:
//Beschreibung:Diese Sammlung von JavaBeans liefert eine grundlegende Bibliothek
//zur Erstellung von TeachWare, welche Animationen verwendet.

package viror.teach.anim;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.JPanel;


public class AnimDisplay extends JPanel implements MouseMotionListener
{
  BorderLayout borderLayout1 = new BorderLayout();

  /* following are some constants that can be used in the
     constuctor to set the background etc. */
  /** paint a square in the background using the components
      foregroundcolour (frames our display) */
  public static final int SQUARE = 1;

  /** The current setting for the background */
  int Style;

  /** grid size in X direction */
  int GridX;
  /** grid size in Y direction */
  int GridY;
  /** scale factor */
  private double Scale;
  /** OffsetX */
  private int OffX;
  /** OffsetY */
  private int OffY;
  /** how much of the screen should the display use? (Percentage) */
  private double ScreenUsage = 0.25;

  /** List of objects to display */
  AnimList List;

  /** The rubber band origin coordinates. Set when activateRubberBand is
   called. */
  private Point RubOrigin = new Point(0,0);
  /** The current mouse position. This is updated as mouse is moved. */
  private Point CurrPos = new Point(0, 0);
  /** True if a rubber band should be displayed. Set by activateRubberBand
   and deavctivateRubberBand. */
  private boolean RubberMode = false;

  public AnimDisplay(int style, int gridX, int gridY, AnimList a)
  {
    Style      = style;
    GridX      = gridX;
    GridY      = gridY;
    setList(a);
    this.addMouseMotionListener(this); // for visual updates only
    this.setBackground(Color.WHITE);
    //this.setDoubleBuffered(true);
  }

  public AnimDisplay(int style, AnimList a)
  {
  this(style, 200, 200, a);
  }

  public AnimDisplay()
    {
    this(SQUARE, new SimpleAnimList());
    }

  /** Sets the AnimList. */
  public void setList(AnimList a)
    {
      if (List != null)
	{
	  List.setDisplay(null); // remove old list
	  this.removeMouseListener(List);
	}
      List = a;
      List.setDisplay(this); // set other way round
      this.addMouseListener(List);
      this.addMouseMotionListener(List);
    }

  /** Sets the grid size. See the manual for details on the grid
    as opposed to the screen coordinate system. */
  public void setGrid(Dimension d)
    {
    GridX = d.width;
    GridY = d.height;
    }

  /** Returns the size of the grid */
  public Dimension getGrid()
    {
    return new Dimension(GridX, GridY);
    }

  /** Sets the percentage of the screen the display should use.
    Note that the effect of this will be limited if it is
    called after the component was shown on screen.
    @param percent The percentage value of how much of the
             screen should be used. */
  public void setScreenUsage(double percent)
    {
    ScreenUsage = percent;
    }

  /** Returns the amount of screen used when at preferredSize */
  public double getScreenUsage()
    {
    return ScreenUsage;
    }

  /** Every time the component size is changed we want to
      recalculate the scaling factor to ensure proper display.
      We also have to calculate an offset in one direction
      to avoid stretching (see documentation for details) */
  public void setBounds(int x, int y, int width, int height)
  {
    double px,py;
    int other = 0;
    px = (width-1) / (1.0 * GridX);
    py = (height-1) / (1.0 * GridY);
    // why -1? to ensure that 'maxCoord' still on screen

    /** only use the more extreme scaling */
    if (px < py)
      {
      // we use the smaller scaling
      Scale = px;
      // calculate offset in the y direction
      // 1. how many pixels does y use at this scale?
      other = (int) Math.round(Scale * GridY);
      // 2. Thus, how many pixels are left unused?
      other = height - other;
      // 3. Offset is half of the unused pixels
      OffY = other / 2;
      // 4. Other offset is 0
      OffX = 0;
      }
    else
      {
      // we use the smaller scaling
      Scale = py;
      // calculate offset in the y direction
      // 1. how many pixels does y use at this scale?
      other = (int) Math.round(Scale * GridX);
      // 2. Thus, how many pixels are left unused?
      other = width - other;
      // 3. Offset is half of the unused pixels
      OffX = other / 2;
      OffY = 0;
      }
  super.setBounds(x,y,width,height);

  // now redo all Elements
  AnimRepresentation[] all = List.getPeers();
  AnimElement[] elem = List.getAll();
  for(int i=0; i < all.length; i++)
    {
    all[i].scale(Scale);
    all[i].setPosition(elem[i].getGridPosition());
    }
  }

  /** this method is called by getPreferredSize, getMinimumSize and
      getMaximumSize. It computes the correct display size for the
      screen at hand. The value is then modified as seen fit. */
  protected Dimension computeSize()
  {
  double use = Math.sqrt(ScreenUsage);
  Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
  size.height *= use;
  size.width  *= use;
  return size;
  }

  /** the preferred size is based on the screen resolution and
      the chosen grid. There is no need to calculate scale here
      as this will be done as soon as the layout manager adjusts
      the size of the component.*/
  public Dimension getPreferredSize()
  {
  int x,y;
  Dimension size = computeSize();
  return size;
  }

  public double getScale()
  {
  return Scale;
  }

  /** the paint method draws all the components in the AnimList
    according to the given scale. */
  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);
    AffineTransform af = new AffineTransform();
    af.setToIdentity();
    int t = getSize().width;
    int r = getSize().height;

    Graphics2D g2 = (Graphics2D) g;

    g2.setTransform(af);
    g2.setPaint(Color.black);

    // draw the rubber band first
    if (RubberMode)
      {
      Line2D.Double l = new Line2D.Double(RubOrigin.getX(), RubOrigin.getY(),
					  CurrPos.getX(), CurrPos.getY());
      g2.draw(l);
      }

    // paint components
    AnimRepresentation[] all =  List.getPeers();
    for(int i=0; i < all.length; i++)
      {
	all[i].paint(g2);
      }

    // paint illegal regions
    Paint temp = g2.getPaint();
    g2.setPaint(SystemColor.menu);
    g2.fill(new Rectangle2D.Float(0,0,OffX, r));
    g2.fill(new Rectangle2D.Float(t-OffX,0,t, r));
    g2.fill(new Rectangle2D.Float(0,0,t,OffY));
    g2.fill(new Rectangle2D.Float(0,r-OffY,t,r));
    g2.setPaint(temp);
    int x=0,y=0,x2=0,y2=0;
    if (Style == SQUARE)
      {
      g2.setPaint(SystemColor.controlShadow);
      Dimension d = convertXY(GridX, GridY);
      x = d.width;
      y = d.height;
      d = convertXY(0,0);
      y2 = d.height;
      x2 = d.width;
      g2.draw(new Rectangle2D.Float(x2,y2,x-x2,y-y2));
      g2.setPaint(temp);
      }
  }

  /** Utility function used to convert grid coordinates into coords
    that can be used with paint. */
  protected Dimension convertXY(int x, int y)
  {
    int rx, ry;
    rx = (int) Math.round(x * Scale);
    ry = (int) Math.round(y * Scale);
    // add offsets
    rx += OffX;
    ry += OffY;
    Dimension d = new Dimension(rx,ry);
    return d;
  }

  protected Dimension convertXY(Dimension d)
    {
    return convertXY(d.width, d.height);
    }

  /** The inverse function to convertXY - it takes actual coordinates
    and returns the corresponding grid ones. Since this is used to
    convert MouseEvent values, the parameter is of type point. */
  protected Dimension unconvertXY(Point p)
    {
    int rx, ry;
    rx = p.x;
    ry = p.y;
    // remove offsets
    rx -= OffX;
    ry -= OffY;
    // unscale
    rx = (int) Math.round(1.0 * rx / Scale);
    ry = (int) Math.round(1.0 * ry / Scale);
    return new Dimension(rx,ry);
    }

  /** Add an element to the display. A shortcut for calling the
    same method in AnimList */
  public void addElement(int layer, AnimElement a)
    {
    if (List != null) List.addElement(layer,a);
    }

  /** Remove an element from the display. */
  public void removeElement(AnimElement a)
    {
    if (List != null) List.removeElement(a);
    }

  /** Activates the rubber band mode. In this mode a line is drawn between
    the point given here as start point and the current mouse position.
    Typical usage is to connect path endpoints. */
  public void activateRubberBand(Point d)
    {
    RubOrigin = d;
    RubberMode = true;
    }

  /** Deactivates the rubber band mode. */
  public void deactivateRubberBand()
    {
    RubberMode = false;
    }

public void mouseMoved(MouseEvent e)
    {
    CurrPos = e.getPoint();
    }

public void mouseDragged(MouseEvent e)
    {
    CurrPos = e.getPoint();
    // if anim is stopped we have to repaint ourselves!
    repaint();
    }


}



