
//Titel:Viror Animationsbibliothek
//Version:
//Copyright:Copyright (c) 1998
//Autor:Ralf Gitzel
//Organisation:
//Beschreibung:Diese Sammlung von JavaBeans liefert eine grundlegende Bibliothek
//zur Erstellung von TeachWare, welche Animationen verwendet.

package viror.teach.anim;

import java.awt.event.*;
import java.awt.Dimension;

/** Contains all the data to be displayed. It is also attached to
  the AnimDisplay as a Listener. Note that implementations of this
  interface should have all methods synchronized that access any
  internal list etc. to avoid exceptions. */
public interface AnimList extends MouseListener, MouseMotionListener
{

static final int NEUTRAL     = 0;
static final int INSERT      = 1;
static final int CONNECT_ONE = 2;
static final int CONNECT_TWO = 3;

/** Returns an array of all peers. Used to draw. Since the only use for
    the return value is to move through it once to draw, we do not use
    a list or any other fancy collection class.
    @return All AnimRepresentations belonging to the objects in the
    list, represented in the very order in which they are to be drawn.*/
AnimRepresentation[] getPeers();

/** adds another element to the list. The layer indicates if element
    will cover others, i.e. if an element is on layer 2, it will cover
    those on layer 1 if they share the same screen position. */
void addElement(int layer, AnimElement e);

/** moves all elements by one "turn" */
void moveAll();

/** This function removes all elements that are "equal" (as in Object.equals())
    to e. While only one element of a kind should be in the list
    some applications might want to have several of the same id.
    @return True is returned if e was in the list, false if nothing
    was changed (since e was not in the list, nothing was deleted) */
boolean removeElement(AnimElement e);

/** completely empties the list of all elements. */
boolean removeAll();

/** gives information on the number of elements in the list.
    @return The number of AnimElements in the list, regardless of
    layer. */
int getSize();

/** Access all AnimElements stored in the List.
    @return Array of AnimElements order according to layer (lowest
    layer first). */
AnimElement[] getAll();

/** Returns an array of all selected elements */
AnimElement[] getAllSelected();


/** Removes all elements currently selected */
boolean removeAllSelected();

/** If two AnimStatics are selected they will be connected by a
  copy ('clone') of the given AnimPath. Otherwise nothing happens.
  The established connection will be duplex, as this is what most people
  will expect. -> This will create two AnimPath objects.
  @returns True is returned if the operation was successful, false otherwise.*/
boolean connectSelected(AnimPath p);

/** Sets the mood of the list. The mood influences what will happen when
  a mouse event occurs. NEUTRAL will pass the event on to the selected
  object, if any. INSERT will add a clone of the object set with
  setInsertCandidate to the animation on the maximum layer.
  Note that some implementations will probably show their mood by
  changing the cursor (which SimpleAnimList does). */
void setMode(int m);

/** Returns the mood of the list. */
int getMode();

/** The specified element will bi inserted if the mouse is clicked
  while in INSERT-Mood. */
void setInsertor(AnimElement e);

/** DO NOT CALL THIS FUNCTION! This function sets a reference to the display
  which is often needed for inital scaling of AnimRepresentations etc. Using
  this function is a bad idea since it does not guarantee integrity (use
  AnimDisplay.setList() instead, which incidentally calls this function).
  Unfortunately it is impossible to make interface functions protected,
  otherwise this one would be. */
void setDisplay(AnimDisplay d);

/** Use this to get the Display associated with the AnimList */
AnimDisplay getDisplay();

/** This method ensures that all objects already in the list before
    the connction to a display class are adjusted to the properties
    of the display. This function is to be called by AnimDisplay only */
void initialize();

/** This method is used to get the shape at the given coordinates
  @param point The coordinates where to look for shapes
  @returns The shape at the given position or null */
AnimElement pick(Dimension point);

}


