
//Titel:        Viror Animationsbibliothek
//Version:
//Copyright:    Copyright (c) 1998
//Autor:       Ralf Gitzel
//Firma:
//Beschreibung:  Diese Sammlung von JavaBeans liefert eine grundlegende Bibliotek
//zur Erstellung von TeachWare, welche Animationen verwendet.

package viror.teach.anim;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/** The basic class for all Peers - i.e. the physical representations
  of the objects in our animation. */
public abstract class AnimRepresentation implements PropertyChangeListener,
  Cloneable
{

  /** A reference to the AnimDisplay used for this animation. Do not
   change this variable, it is set by the List when adding an element.*/
  AnimDisplay Display;



  /** If true, this component is not displayed */
  protected boolean Cloaked;

  /** If true, the object is modified in such a way that it stands
    out from the others of its type */
  protected boolean Highlighted;

  /** The current scale factor. */
  protected double SFactor;

  /** The bounding box for this object. */
  protected Rectangle2D BoundingBox;


  /** Set up infrastructure for listeners etc. */
  public AnimRepresentation()
    {
    Cloaked = false;
    Highlighted = false;
    }

  /** This is fired when the coordinates of the AnimElement change. */
  public void propertyChange(PropertyChangeEvent e)
    {
    if(!e.getPropertyName().equals("GridPosition")) return;
    Dimension d = (Dimension) e.getNewValue();
    if (Display != null)
      {
	setPosition(d);
      }
    }

  /** Set the onscreen position of this Representation. */
  protected void setPosition(int x,int y)
    {
    setPosition(new Dimension(x,y));
    }

  /** Set the onscreen position of this Representation. Note that
      this corresponds to the MIDDLE of the object and not its
      upper corner! */
  public abstract void setPosition(Dimension d);

  /** When this returns true, the object is invisible on screen */
  public boolean isCloaked()
    {
    return Cloaked;
    }

  /** Set true to make object invisible on screen */
  public void setVisible(boolean c)
    {
    Cloaked = c;
    }

  /** This is called in AnimDisplay.paintComponent to draw exactly
      this object. */
  public abstract void paint(Graphics2D g);

  /**  Change the size of this element. To fit on screen */
  public abstract void scale(double factor);

  /** Do not call this method! It is used exclusively by AnimDisplay
    when a list is added to it. Then all elements are set to the new
    display and have their positions etc. updated */
  protected void setDisplay(AnimDisplay d)
    {
    Display = d;
    scale(d.getScale());
    }

  /** returns the display this AnimRepresentation uses. */
  public AnimDisplay getDisplay()
    {
    return Display;
    }

  /** Test whether this object <i>might</i> have been selected. A bounding
    box encompasses the whole object, so if the point is not within
    the bounding box we do not have to start the more complex intersection
    test.
    @param point The point that is to be examined
    @returns the return value is true if 'point' is inside this
       objects bounding box. */
  public boolean testBoundingBox(Dimension point)
    {
    if (BoundingBox == null)
      {
      return true; // we need the exact check since it might still be hit!
      }
    return BoundingBox.contains(point.width, point.height);
    }

  /** Tests whether a point is "on" this object, i.e. whether it has
    been selected or not. Note that testBoundingBox is usually called before
    this to accelerate picking. */
  public abstract boolean testActual(Dimension point);

  /** Updates the bounding box surrounding this object. If a certain type
    of object is not to have one, the implementation for this function should
    do nothing - this will seriously impair our acceleration however!
    The bounding box should be updated whenever scale or position are
    changed. */
  protected abstract void updateBoundingBox();

  /** makes this object stand out from the others of its type.
    This may be done in several ways - framing it, changing colour etc.
    This is usually used to show that an object has been selected.
    @param high If high is true, the element will be highlighted otherwise
    it will return to normal. */
  public void setHighlighted(boolean high)
    {
      Highlighted = high;
      if (Display != null) Display.repaint(); // redo immediately
   }

  /** returns the value of highlighted. */
  public boolean isHighlighted() { return Highlighted; }

  public Object clone() throws CloneNotSupportedException
    {
    AnimRepresentation a = (AnimRepresentation) super.clone();
    a.BoundingBox = null; // get your own BBox...
    return a;
    }

}









