
//Titel:        Viror Animationsbibliothek
//Version:
//Copyright:    Copyright (c) 1998
//Autor:       Ralf Gitzel
//Firma:
//Beschreibung:  Diese Sammlung von JavaBeans liefert eine grundlegende Bibliothek
//zur Erstellung von TeachWare, welche Animationen verwendet.

package viror.teach.anim;

import java.io.IOException;
import java.beans.PropertyChangeEvent;
import java.awt.image.BufferedImage;
import java.awt.Image;
import java.awt.Dimension;
import java.awt.event.MouseEvent;

public class AnimStatic extends AnimElement
{


  /** The standard constructor creates a static element that is
    represented by a square and that is dragable. Coordinates are
    0,0 */
  public AnimStatic()
    {
    super(VectorRep.createPrimitive(VectorRep.SQUARE, 10), 0,0);
    Dragable = true;
    }

  /** This constructor creates a static element that is represented by
      a bitmap - a situation that occurs 99% of the time.
      @param img The bitmap to be used.
      @param drag When true, the element can be dragged on the display
      by holding the mouse button.
      @param x The x position in the grid
      @param y The y position in the grid */
  public AnimStatic(Image img, boolean drag, int x, int y)
    {
    super(new BitmapRep(img),x,y);
    Dragable = drag;
    }

  /** This constructor creates a static element that is represented by
      a bitmap. Unlike the previous one, it also defines an image to
      be shown when item is selected.
      @param img The bitmap to be used.
      @param sel The bitmap to be shown when item is selected.
               It is <i>highly recommended</i> that this image is the same
	       size as the unselected one.
      @param drag When true, the element can be dragged on the display
      by holding the mouse button.
      @param x The x position in the grid
      @param y The y position in the grid */
  public AnimStatic(Image img, Image sel, boolean drag, int x, int y)
    {
    super(new BitmapRep(img, sel),x,y);
    Dragable = drag;
    }


  /** This constructor creates a static element that has a polygon
    as visual component.
    @param nodes The list of nodes that define the polygon.
    @param open  If open is false, the polygon will have the first
                 and last node connected and will be filled.
		 (For unfilled closed polygons set node[end] = node[0]
		 and open = true)
    @param drag If true, the element can be dragged on the screen. */
  public AnimStatic(Dimension[] nodes, boolean open,
		    boolean drag)
    {
    super (VectorRep.createPolygon(nodes, open),
               nodes[0].width, nodes[0].height);
    Dragable = drag;
    }

  public void move()
  {
  // AnimStatics do not move so nothing happens here....
  }

  /** If set to true, the element can be dragged around the
    screen. */
  public void setDragable(boolean drag)
    {
    Dragable = drag;
    }

  /** This method is called to inform the AnimStatic that it has been connected
   *  to a path. This may result in change of the peer, updating of internal lists
   *  etc. By default the method is empty. */
  protected void connect(AnimPath l)
    {
    }

  /** This method is called to inform the AnimStatic that it has been disconnected
   *  from a path. This may result in change of the peer, updating of internal lists
   *  etc. By default the method is empty. */
  protected void disconnect(AnimPath l)
    {
    }

  /** Use this method for debugging and other output */
  public String toString()
    {
    String ret = new String("AnimStatic at" + GridX + " " +
			    GridY + "  Drag=" + Dragable + " Type: "+ getIdentification());
    return ret;
    }


}
