
//Titel:        Viror Animationsbibliothek
//Version:
//Copyright:    Copyright (c) 2000
//Autor:       Ralf Gitzel and Adriaan Geursen (testActual())
//Firma:
//Beschreibung:  Diese Sammlung von JavaBeans liefert eine grundlegende Bibliothek
//zur Erstellung von TeachWare, welche Animationen verwendet.

package viror.teach.anim;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.image.*;
import java.io.IOException;

/** Images passed to this class should be fully loaded already.
  Use the MediaTracker class to ensure this. */
public class BitmapRep extends AnimRepresentation implements ImageObserver
{

  /** The original unscaled image. This value remains unchanged
   all the time, when scaling etc. is done the copy 'Current' is
   changed. This avoids loss of quality over multiple transforms */
  protected final BufferedImage Original;

  /** This variable is used to store the image information as it
    is loaded asynchronously. As soon as all is there we do not
    need it any more */
  private Image Loader;

  /** The scaled version of the picture. Very important: never set
      Current = Original or sth. like that - this will give us
      two references to the same object and that is NOT what we
      want. */
  protected BufferedImage Current;

  /** This graphic will be displayed when the object is selected.
    When Selected is null, a frame will be drawn around the graphic
    to show that it is selected. Note that the class implicitly
    assumes that the Selected image is the same size as the original
    one - this will affect picking and positioning. */
  protected BufferedImage Selected;

  /** The scaled version of the picture for selected graphics. This
    will be displayed when a graphic is selected */
  protected BufferedImage SCurrent;

  /** This member is used to determine the scaling quality. */
  protected RenderingHints Quality;

  /** This value allows to give pictures an inital scale, i.e. if
    IniScale is .5 and we scale to .5 the picture will be .25 of
    its original size. While this might seem useless at first glance
    it allows to have higher resolution bitmaps at a reduced size such
    that enlarging the display will not cause them to look jagged */
  protected double IniScale;

  /** the x 'real' coordinate of the upper left corner */
  protected int X;

  /** the y 'real' coordinate of the upper left corner */
  protected int Y;


  /** Modulo value for getPosition (see there). It ensures
      that the getPosition value is the same as the one
      entered via setPosition. */
  private int ModX;

  /** Modulo value for getPosition (see there). It ensures
      that the getPosition value is the same as the one
      entered via setPosition. */
  private int ModY;

  /** A cheapo  constructor. Note that the image has to be
   fully loaded by the time we reach this constructor, otherwise the
   bitmaps will be incomplete. */
  public BitmapRep(Image temp)
  {
  IniScale = 0.5;
  Quality = new RenderingHints(RenderingHints.KEY_ANTIALIASING,
			       RenderingHints.VALUE_ANTIALIAS_ON);
  int width = 0, height = 0;
  while (width < 1 || height < 1)
    {
    width = temp.getWidth(this);
    height = temp.getHeight(this);
    }
  Loader = temp;
  Original = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
  Graphics2D big = Original.createGraphics();
  big.drawImage(Loader, 0, 0, this);
  scale(1.0);
  }

  /** This constructor sets a graphic to be shown when item is selected.
    @param original The image to be shown when unselected
    @param selected The image to be shown when selected */
  public BitmapRep(Image original, Image selected)
    {
    this(original);
    setSelectedImage(selected);
    scale(1.0);
    }



  public void setPosition(Dimension d)
  {
    Dimension d2 = null;
    if (Display != null) d2 = Display.convertXY(d);
    else d2 = (Dimension) d.clone();
    // calculate the upper left corner coordinates
    // since scale is called before setPosition in
    // a resize, we can expect these values to be ok
    X = d2.width  - Current.getWidth()  / 2;
    Y = d2.height - Current.getHeight() / 2;
    ModX =  Current.getWidth()  % 2;
    ModY = Current.getHeight()  % 2;
    updateBoundingBox();
  }

  /** Returns the position of the Bitmap. */
  public Dimension getPosition()
  {
    Dimension d = new Dimension();
    // recalculate the center coordinates
    d.width  = X + ModX + Current.getWidth()  * 2;
    d.height = Y + ModY + Current.getHeight() * 2;
    return d;
  }

  /** This method can be used to set a graphic that should be displayed
    when the element is selected. If this value is null, selection will
    be indicated by a border around the object. */
  public void setSelectedImage(Image select)
    {
    int width = 0, height = 0;
    while (width < 1 || height < 1)
      {
      width = select.getWidth(this);
      height = select.getHeight(this);
      }
    Selected = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
    Graphics2D big = Selected.createGraphics();
    big.drawImage(select, 0, 0, this);
    }


  public void paint(Graphics2D g)
  {
    if (!Cloaked)
      {
	if (Selected == null)
	  {
	    g.drawImage(Current,null, X,Y);
	    if (Selected == null && Highlighted)
	      {
		// mark by painting a 3DFrame around it
		Paint temp = g.getPaint();
		g.setPaint(Color.gray);
		g.draw3DRect((int) BoundingBox.getX(), (int) BoundingBox.getY(),
			     (int) BoundingBox.getWidth(),
			     (int) BoundingBox.getHeight(), true);
		g.setPaint(temp);
	      }
	  }
	if (Selected != null)
	  {
	    if (!Highlighted)
	      {
	      g.drawImage(Current,null, X,Y);
	      }
	    else
	      {
	      g.drawImage(SCurrent, null, X,Y);
	      }
	  }
      }
  }

  public void scale(double factor)
  {
    // first check if a rescale is really necessary
    if (factor > SFactor - 0.05 && factor < SFactor + 0.05)
      {
      // A rescale is not required
      // and no update of SFactor required since it is only
      // for size and not for position
      return;
      }
    SFactor = factor; // save for future acceleration
    double fac = factor * IniScale;
    int high = (int) (Original.getHeight() * fac);
    int wide = (int) (Original.getWidth()  * fac);
    Current = new BufferedImage(wide, high ,BufferedImage.TYPE_INT_ARGB);
    AffineTransform trans = new AffineTransform(); // identity matrix
    trans.scale(fac, fac); // make it a scaling matrix
    AffineTransformOp scale = new AffineTransformOp(trans, Quality);
    scale.filter(Original, Current);
    updateBoundingBox();
    // now update the SCurrent as well
    if (Selected == null) return;
    SCurrent = new BufferedImage(wide, high, BufferedImage.TYPE_INT_ARGB);
    scale.filter(Selected, SCurrent);
  }

  /** The initial scale. Set this value to make pictures bigger (>1)
    or smaller (<1) than they should be at the given scale. */
  public void setInitialScale(double ini) { IniScale = ini; }

  /** Returns the current value of InitialScale */
  public double getInitialScale() { return IniScale; }


  /** This function only ensures that pics have to be loaded
   completely before being used in this class. Asynchronous loading
   will cause erratic behaviour since we copy pictures before we scale
   them etc. */
  public boolean imageUpdate(Image img, int infoflag, int x,
			     int y, int width, int height)
    {
      System.err.println("Image has to be loaded completely before using it with");
      System.err.println("BitmapRep! If you are already using a media tracker, the");
      System.err.println("cause may be a wrong URL for the picture.");
      return false;
    }

  /** The bounding box of a bitmap is very simple - basically we
    take its x,y coordinates and its height and width */
  protected void updateBoundingBox()
    {
      BoundingBox = new Rectangle2D.Double(X,Y,Current.getWidth(),
					   Current.getHeight());
    }



/** This method tests the pixel's alpha value to see whether it is part of
 *  the graphic or not.  */
public boolean testActual(Dimension p)
    {
      try
      {
        if ((Current.getAlphaRaster().getPixel((p.width - this.X),
          (p.height - this.Y), new int[1]))[0] == 0) return false;
        return true;
      } catch (Exception e) {
        return true;
      }
    }

  /** Sets the initial scale, i.e. when we have a high resolution grid you
   *  might want to set this value greater than one to make your bitmaps appear
   *  bigger. Do not confuse this with the scale method which rescales the
   *  bitmaps according to resizes.
   *  To describe it mathematically the scale factor for a bitmap at any time
   *  is scale * IniScale.
   */
  public void setIniScale(double i)
    {
    IniScale = i;
    }

  /** Gets the initial scale, i.e. when we have a high resolution grid you
   *  might want to set this value greater than one to make your bitmaps appear
   *  bigger. Do not confuse this with the scale method which rescales the
   *  bitmaps according to resizes.
   *  To describe it mathematically the scale factor for a bitmap at any time
   *  is scale * IniScale.
   */
  public double getIniScale()
    {
    return IniScale;
    }
}





