

package viror.teach.anim;

import java.awt.*;
import java.awt.geom.*;


/** This class draws a polygon which can be either open or
    closed. Closed polygons can be filled or unfilled. */
public class PolygonRep extends VectorRep
{

/** An open polygon */
public static final int OPEN = 0;

/** A closed polygon */
public static final int CLOSED = 1;

/** The Nodes that define the Polygon. These values are redundant
  but are stored for convenience sake to facilitate the
  picking checks. Note that the startpoint is 0/0. As with the
  other Vector shapes, position and size are defined by the Pos
  and Scale matrices. */
protected Dimension[] Node;


/** The waypoints define the look of the polygon.
 @param waypoints The nodes of the Polygon
 @param closed If closed is true the polygons first
        and last node will be connected and it will be filled.
	(To have an unfilled closed polygon simply give the
	first and last point the same coordinates and
	set closed to false) */
public PolygonRep(Dimension[] waypoints, boolean closed)
  {
  // set fill
  DoFill = closed;
  /* make a copy of the waypoints - Node = waypoints would
     simply copy reference and we would have a call by
     reference and not by value */
  Node = new Dimension[waypoints.length];
  for (int j=0; j < Node.length; j++)
    Node[j] = (Dimension) waypoints[j].clone();
  // now 'translate' to 0/0
  int x = Node[0].width, y = Node[0].height;
  for (int i=0; i < Node.length; i++)
    {
    Node[i].width  -= x;
    Node[i].height -= y;
    }
  updatePolygon();
  setPosition(new Dimension(x,y));
  }

/** Recalculate the shape from the data of Node and doFill */
protected void updatePolygon()
    {
    Original = new GeneralPath();
    ((GeneralPath) Original).moveTo(0,0);
    // use Node information
    for(int i=0; i < Node.length; i++)
      {
      ((GeneralPath) Original).lineTo(Node[i].width, Node[i].height);
      }
    }

/** we test if we are inside or outside of the polygon  */
public boolean testActual(Dimension p)
    {
      //test if nodes are defined
      if (Node == null) return false;

      //epsilon to check if double equals 0
      double epsilon = 0.000000001;

      Point ps       = new Point ((int) Math.round(p.getWidth()),
                      (int) Math.round(p.getHeight()));
      Dimension pg   = Display.unconvertXY(ps);
      //compute the translated coordinates
      ps             = new Point ((int)
                  Math.round(Pos.getTranslateX()) ,(int)
                  Math.round(Pos.getTranslateY()));
      Dimension tran = Display.unconvertXY(ps);
      //from here all values relative to grid (display independent)...
      //clickY is our scanline
      double clickX = pg.getWidth()  - tran.getWidth();
      double clickY = pg.getHeight() - tran.getHeight();

      //check if all nodes are set corectly
      int nlength = -1;
      for (int i=0; i < Node.length; i++) {
        if (Node[i] == null) break;
        else nlength++;
      }
      //we need tline to test if scanline crosses the lined between the nodes
      Line2D.Double tline;
      //if polygone has only 1 or 2 nodes (nlength = 0 or 1) we need special processing
      if (nlength == 0) return false;
      //minX is the lowest X value there is
      double minX = 0;
      for (int i=0; i < nlength; i++) if (Node[i].getWidth() < minX)
                                             minX = Node[i].getWidth();
      if (nlength == 1) {
        tline = new Line2D.Double(Node[0].getWidth(),
            Node[0].getHeight(),Node[1].getWidth(), Node[1].getHeight());
        //test if line intersects scanline
        return tline.intersects(minX-1, clickY, clickX, clickY);
      }
      //run through the nodes to check if they cross the scanline
      //inside mod 2 returns weather we are inside or outside the polygone
      int inside = 0;
      Dimension pre, act, suc;
      for (int i=0; i <= nlength; i++) {
        if (i == 0) {
          pre = Node[nlength];
          act = Node[i];
          suc = Node[i+1];
        }
        else {
          if (i == nlength) {
            pre = Node[i-1];
            act = Node[i];
            suc = Node[0];
          }
          else {
            pre = Node[i-1];
            act = Node[i];
            suc = Node[i+1];
          }
        }
        //we need to check for 3 cases for the line from act to suc
        //case 1 line is above or below the scanline
        if (((act.getHeight() > clickY) && (suc.getHeight() >
            clickY)) || ((act.getHeight() < clickY) && (suc.getHeight() <
            clickY)))
        {
          //nothing to do...
        }
        else
        {
          //case 2 one point over and one below the scan line
          if (((act.getHeight() < clickY) && (suc.getHeight() >
               clickY)) || ((act.getHeight() > clickY) && (suc.getHeight() <
              clickY))) {
            //if lines intercrsect we increse inside by one
            if (Line2D.linesIntersect(act.getWidth(),
            act.getHeight(), suc.getWidth(), suc.getHeight(), minX-1, clickY,
            clickX, clickY)) inside++;
          }
          //case 3 at least on is on scanline
          else
          {
            //now we need to check 3 other cases
            //the line starts outside and ends on the scanline
            if ((act.getHeight() != clickY) && (suc.getHeight() -
              clickY < epsilon) && (suc.getWidth() < clickX)) {
              inside++;
            }
            else
            {
            //the line ends outside and starts on the scanline
              if ((suc.getHeight() != clickY) && (act.getHeight() -
                  clickY < epsilon) && (act.getWidth() < clickX)) {
                //if pre and suc are on same side of scanline we increase inside by one
                if (((pre.getHeight() < clickY) && (suc.getHeight() <
                    clickY)) || ((pre.getHeight() > clickY) && (suc.getHeight() >
                    clickY))) inside++;
              }
            //the line is on the scanline
            //nothing to do...
            }
          }
        }
      }
      return ((inside % 2) == 1);
    }
}
