
package viror.teach.anim;

import javax.swing.*;
import javax.swing.plaf.basic.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.Customizer;

/** This class defines a JPanel with the standard buttons used
  to manipulate the Animation. The buttons are: Start, Stop,
  InsertX, Delete, Connect, View, Modify. Text for these can be changed */
public class StandardButtons extends JPanel
{

/** The AnimList where the information is stored */
private AnimList AList;

/** The animation control class */
AnimCycle AnimControl;

/** The display where we show the animation */
private AnimDisplay Display;

/** The class used to connect elements. */
private AnimPath ConnectType;

/** The button that starts the animation */
protected JButton Start;

/** The button that stops the animation */
protected JButton Stop;

/** The button that deletes the selected elements */
protected JButton Delete;

/** The button that deletes the selected elements */
protected JButton Connect;

/** The button that deletes the selected elements */
protected JButton View;

/** The button that deletes the selected elements */
protected JButton Modify;

/** This label displays information for the benefit of the user. */
protected JLabel Message;

/** This message will be written when waiting for orders. */
protected String ReadyText = "Ready";

/** The JPanel where insert buttons are added */
JPanel Lower;

/** The Frame where additional info is displayed */
JFrame Viewer;

/** The button to close the viewer window */
JButton CloseV;

/** The button that undoes changes in the modify window */
JButton CancelV;

/** This ActionListener handles insertions */
ActionListener InsertListener;

/*** Selection error message. */
protected String SelectErrorText =
  "Either nothing or insufficient elements selected.";

/** This (Hash)Map maps insertion buttons to elements that will be inserted
  when pressing the button. This architecture is necessary since
  since we do not know which kinds of classes the upper library will have */
Map Insertion;

  /////////// Constructors /////////////////

/** The standard constructor sets up the basics but has to be augmented by
  the sets and gets (especially the AList should be set) */
public StandardButtons()
    {
    setConnectType(new AnimPath());
    Insertion = new HashMap();
    Start     = new JButton("Start");
    Stop      = new JButton("Stop");
    Delete    = new JButton("Delete");
    Connect   = new JButton("Connect");
    View      = new JButton("View");
    Modify    = new JButton("Modify");
    Message   = new JLabel(ReadyText);

    // init listeners
    InsertListener = new SBModeHandler();
    SBActOnHandler acton = new SBActOnHandler();
    Delete.addActionListener(acton);
    Connect.addActionListener(acton);
    Start.addActionListener(InsertListener);
    Stop.addActionListener(InsertListener);
    View.addActionListener(acton);
    Modify.addActionListener(acton);

    // do display stuff
    setLayout(new GridLayout(3,1));
    JPanel upper = new JPanel();
    // Lower already defined - used for insert buttons
    Lower = new JPanel();
    JPanel mboard = new JPanel();
    Color high  = SystemColor.controlHighlight;
    Color lhigh = SystemColor.controlLtHighlight;
    Color shw   = SystemColor.controlShadow;
    Color dkshw = SystemColor.controlDkShadow;
    mboard.setBorder(new BasicBorders.FieldBorder(shw, dkshw, high, lhigh));
    upper.add(Start);
    upper.add(Stop);
    upper.add(Delete);
    upper.add(Connect);
    Lower.add(View);
    Lower.add(Modify);
    mboard.setLayout(new BorderLayout());
    mboard.add(Message, BorderLayout.CENTER);
    add(upper);
    add(Lower);
    add(mboard);

    }

  //////// methods //////////////

  //////// sets and gets ////////////

/** Adds a new button to insert the specified type of element.
  It is not allowed to remove buttons lateron.
 @param text The button text.
 @param elem The element that will be inserted. */
public void addInsertButton(String text, AnimElement elem)
    {
    JToggleButton ins = new JToggleButton(text);
    Insertion.put(ins, elem);
    Lower.add(ins);
    ins.addActionListener(InsertListener);
    }


/** sets the text for the start button */
public void setStartText(String start)
    {
    Start.setText(start);
    }

/** sets the text for the stop button */
public void setStopText(String stop)
    {
    Stop.setText(stop);
    }

/** sets the text for the stop button */
public void setDeleteText(String delete)
    {
    Delete.setText(delete);
    }

/** sets the text for the start button */
public void setConnectText(String connect)
    {
    Connect.setText(connect);
    }

/** the class that will be used to connect the two elements.
  The given class will be cloned for each connect, so
  its 'default' parameters should be chosen carefully. */
public void setConnectType(AnimPath p)
    {
    ConnectType = (AnimPath) p.clone();
    }

/** sets the text for the start button */
public void setViewText(String view)
    {
    View.setText(view);
    }

/** sets the text for the start button */
public void setModifyText(String modify)
    {
    Modify.setText(modify);
    }

/** sets the text that will be shown when waiting for orders. Default
  value is "Ready". */
public void setReadyText(String ready)
    {
    ReadyText = ready;
    }

/** sets the text that will be shown when a button was pressed that
  required something to be selected but that was nothing selected.
  Default value is "Either nothing or insufficient elements selected.". */
public void setSelectErrorText(String selerr)
    {
    SelectErrorText = selerr;
    }



/** Sets the list which contains the elements. Via this the display is
  set as well. */
public void setAList(AnimList l) throws NullPointerException
    {
    Display = l.getDisplay();
    if (Display == null) throw new NullPointerException(
      "Display of AnimList must be set to add StandardButtons!");
    AList = l;
    }

/** Sets the class that controls the animation cycle */
public void setAnimControl(AnimCycle c)
    {
    AnimControl = c;
    }


  ////////// inner classes ///////////

/** this class handles all buttons that will do something with
  the objects already selected. The Delete and Connect buttons form
  such a category. */
class SBActOnHandler implements ActionListener
  {
  public void actionPerformed(ActionEvent e)
    {
    boolean ret = false;
    if (e.getSource() == Delete) ret = AList.removeAllSelected();
    if (e.getSource() == Connect) ret = AList.connectSelected(ConnectType);
    if (e.getSource() == View)
      {
	viewItem(AList.getAllSelected());
	return;
      }
    if (e.getSource() == Modify)
      {
      modifyItem(AList.getAllSelected());
      return;
      }

    // set message after action
    if (!ret) Message.setText(SelectErrorText);
    else Message.setText(ReadyText);

    // finally untoggle all insert buttons
    Iterator i = Insertion.keySet().iterator();
    while (i.hasNext())
      {
      ((JToggleButton) i.next()).setSelected(false);
      }
    // and set mood to neutral
    if (AList.getMode() == AList.INSERT) AList.setMode(AnimList.NEUTRAL);
    }

  public void viewItem(AnimElement[] all)
    {
    if (all.length != 1)
      {
      Message.setText(SelectErrorText);
      return;
      }
    JComponent c = all[0].getCustomizer(false);
    if (c == null)
      {
	Message.setText(all[0].toString());
      }
    else
      {
	// set up viewer window
	Viewer = new JFrame("-=*=-");
	Container cont = Viewer.getContentPane();
	cont.setLayout(new BorderLayout());
	JPanel butt = new JPanel();
	CloseV = new JButton("OK");
	butt.add(CloseV);
	cont.add(butt, BorderLayout.SOUTH);
	CloseV.addActionListener(new ViewerHandler((UndoCustomizer) c));
	Viewer.setVisible(false);
	Viewer.getContentPane().add(c, BorderLayout.CENTER);
	Viewer.pack();
	Viewer.setVisible(true);
      }
    }

  public void modifyItem(AnimElement[] all)
    {
    if (all.length != 1)
      {
      Message.setText(SelectErrorText);
      return;
      }
    JComponent c = all[0].getCustomizer(true);
    if (c == null)
      {
	Message.setText(all[0].toString());
      }
    else
      {
	// set up viewer window
	Viewer = new JFrame("-=*=-");
	Container cont = Viewer.getContentPane();
	cont.setLayout(new BorderLayout());
	JPanel butt = new JPanel();
	CloseV  = new JButton("OK");
	CancelV = new JButton("Cancel");
	butt.add(CloseV);
	butt.add(CancelV);
	cont.add(butt, BorderLayout.SOUTH);
        ViewerHandler vh = new ViewerHandler((UndoCustomizer) c);
	CloseV.addActionListener(vh);
        CancelV.addActionListener(vh);
	Viewer.setVisible(false);
	Viewer.getContentPane().add(c, BorderLayout.CENTER);
	Viewer.pack();
	Viewer.setVisible(true);
        AnimControl.suspendAnim();
      }
    }
  }

/** this class handles all buttons that will do something that does
  not require selection. Insert buttons fit this well, as do start and stop. */
class SBModeHandler implements ActionListener
  {
  public void actionPerformed(ActionEvent e)
    {
      // check whether it was any of the insert buttons
      AnimElement ins = (AnimElement) Insertion.get(e.getSource());
      if (ins != null)
	{
	// check one - select or unselect
	if (((JToggleButton) e.getSource()).isSelected())
	  AList.setMode(AnimList.INSERT);
	else
	  {
	    AList.setMode(AnimList.NEUTRAL);
	  }
	// an insert button was pressed
	AList.setInsertor(ins);
	}
      // check for start/stop buttons
      if (e.getSource() == Start) AnimControl.resumeAnim();
      if (e.getSource() == Stop)  AnimControl.suspendAnim();

      // untoggle buttons
      Iterator i = Insertion.keySet().iterator();
      while (i.hasNext())
	{
	JToggleButton butt = (JToggleButton) i.next();
	if (e.getSource() != butt)
	  butt.setSelected(false);
	}
    }
  }

class ViewerHandler implements ActionListener
  {

  UndoCustomizer Cuss;

  public ViewerHandler(UndoCustomizer c)
    {
      Cuss = c;
    }

  public void actionPerformed(ActionEvent e)
    {
    Viewer.setVisible(false);
    if (e.getSource() == CancelV) Cuss.undo();
    AnimControl.resumeAnim();
    }
  }

}


