

package viror.teach.anim.net;

import viror.teach.anim.*;
import java.beans.*;

/** This is the base class for all AnimPaths that connect Routers,
 *  SoftwareComponents etc. */
public abstract class AbstractLink extends AnimPath
{

/** The time it takes a bit to travel through this line.
  This is <i>not</i> the time it takes a package to arrive
  though - bandwidth figures into that as well (see there) */
protected int Delay;

///////////////////////////////////////////////

/** Reacts to the arrival of a package by passing it to the router at the
  correct end. If the AnimElement is not a Package it will be ignored */
public void handleArrival(AnimActor e)
    {
    if (!(e instanceof Package)) return; // ignore other actors
    System.out.println("AbstractLink: Package arrives as "+e);
    // case 1: package arrives at start node
    if (e.getCompletion() == 0)
      {
      if (!(getStart() instanceof AbstractRouter)) return;
      ((AbstractRouter) getStart()).packageArrived((Package) e);
      }
    // case 2: package arrives at end node
    if (e.getCompletion() == e.getTravelTime())
      {
      if (!(getEnd() instanceof AbstractRouter)) return;
      ((AbstractRouter) getEnd()).packageArrived((Package) e);
      }
    // otherwise the package is irregular and not handed to any router
    return;
    }


/** Sends a package p from s to the other end of the link.
  @param s The end of the link where the message originates.
           All other routers will have no effect!
  @param p The package to be send. */
public void placePackage(AbstractRouter s, Package p)
    {
    if ((s != getStart()) && (s != getEnd()))
      {
	System.err.println(s + " is no endpoint of " + this);
	return;
      }
    // case startpoint
    if (s == getStart())
      {
      p.setCompletion(0);
      p.setTravelTime(Delay);
      p.setArrivalState(Delay);
      p.setStep(1);
      }
    else
      {
      p.setCompletion(Delay);
      p.setTravelTime(Delay);
      p.setArrivalState(0);
      p.setStep(-1);
      }
    // add the actor to the AnimPath
    addActor(p);
    }

/** Sets the Delay of this link. Delay is the number of animation cycles
  it takes the package to arrive. Usually the values will be unrealistic
  for viewing ease. */
public void setDelay(int d) throws PropertyVetoException
    {
    if (d < 0)
      throw new PropertyVetoException("Positive only!",
	new PropertyChangeEvent(this, "Delay", new Integer(Delay),
				new Integer(d)));

    Delay = d;
    }

/** Gets the Delay of this link. Delay is the number of animation cycles
  it takes the package to arrive. Usually the values will be unrealistic
  for viewing ease. */
public int getDelay()
    {
    return Delay;
    }




}