
package viror.teach.anim.net;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.BevelBorder;
import viror.teach.anim.*;

public class HighlightText extends JPanel implements ProgressIndicator
{

/** The font size used in the display. */
private int FontSize = 12;

/** The maximum font width */
private int FontMaxWidth = 5;

/** The lines of code to be displayed in the window. */
private String[] Line;

/** The scrollbar to view all text */
private JScrollBar Scroll;

/** The JPanel where the text is displayed */
private JPanel TextZone;

/** The visible line on the screen */
  private JLabel[] VLine;

/** The current offset (i.e. the number of lines before
  the first visible one that have been scrolled out) */
private int Offset;

/** The number of lines that should be visible on the screen */
private int VNum = 8;

/** The number of characters in one line (approximately, since characters
    might not be mono-spaced */
private int VCNum = 40;

/** The line that is currently executed */
private int ActiveLine = 0;

/** The colour used to highlight the current line */
Color HiLite = Color.red;

/** The colour used normally */
Color Basic = Color.black;

  //////////////////////

public HighlightText()
    {
    Line    = new String[1];
    Line[0] = "";
    setUpGUI();
    }

private void setUpGUI()
    {
    setLayout(new BorderLayout());
    setBorder(new BevelBorder(BevelBorder.LOWERED));
    Scroll = new JScrollBar(JScrollBar.VERTICAL);
    Scroll.addAdjustmentListener(new HLTScroll(this));
    add(Scroll, BorderLayout.EAST);
    TextZone = new JPanel(new GridLayout(VNum,1));
    // set preferredSize with font specifics
    FontSize = TextZone.getFont().getSize();
    int height = FontSize * VNum;
    int width = FontMaxWidth * VCNum;
    TextZone.setPreferredSize(new Dimension(width, height));
    TextZone.setBackground(Color.white);
    add(TextZone, BorderLayout.CENTER);
    // set up text lines
    VLine = new JLabel[VNum];
    for(int i=0; i < VNum; i++)
      {
      VLine[i] = new JLabel();
      TextZone.add(VLine[i]);
      }
    }



  //////////////////////

/** The variable i is ignored since we only have one progress. */
public void advance(int i)
    {
    if (ActiveLine < Line.length - 1) ActiveLine++;
    if (ActiveLine - Offset >= VNum)
      {
      Scroll.setValue(Scroll.getValue() + VNum / 2);
      }
    updateLine();
    }

/** The variable i is ignored */
public void stepBack(int i)
    {
    if (ActiveLine > 0) ActiveLine--;
    if (ActiveLine < Offset)
      {
      Scroll.setValue(Scroll.getValue() - VNum / 2);
      }
    updateLine();
    }

private void updateLine()
    {
    for (int i = 0; i < VNum; i++)
      {
      if (i+Offset < Line.length)
	{
	VLine[i].setText(Line[i+Offset]);
        VLine[i].setToolTipText(Line[i+Offset]);
	}
      else
	{
	VLine[i].setText("");
	}
      VLine[i].setForeground(Basic);

      }
    if (ActiveLine - Offset >= 0 && ActiveLine - Offset < VNum)
      VLine[ActiveLine - Offset].setForeground(HiLite);
    }

/** Sets the text of this Code Viewer. Each line is its own
  string */
public void setLines(String[] l)
    {
    Line = l;
    Scroll.setMaximum(l.length+1);
    updateLine();
    }

  /** The parameter 'viewer' is ignored -> it is required due to the generic
   *  nature of the interface. */
  public void setPosition(int viewer, int i)
  {
  ActiveLine = i;
  int temp = 0;
  if (ActiveLine - Offset >= VNum || ActiveLine < Offset)
    {
    if (ActiveLine - VNum/2 >= 0) temp = ActiveLine - VNum/2;  // show 'context' when scrolling
    else temp = 0;  // we cannot scroll back any farther than 0
    Scroll.setValue(temp);
    }
  updateLine();
  }

  public int getPosition(int i)
  {
  return ActiveLine;
  }

  /** Set the number of visible lines of text. */
  public void setNumberOfVisibleLines(int vnum)
    {
    VNum = vnum;
    }

  /** Returns the number of visible lines of text */
  public int getNumberOfVisibleLines()
    {
    return VNum;
    }

  /** Sets the number of visible characters in one line */
  public void setNumberOfVisibleCharacters(int vcnum)
    {
    VCNum = vcnum;
    }

  /** Returns the number of visible characters in one line */
  public int getNumberOfVisibleCharacters()
    {
    return VCNum;
    }


  ////////////////

class HLTScroll implements AdjustmentListener
{

private HighlightText Obj;

public void adjustmentValueChanged(AdjustmentEvent e)
    {
    Obj.Offset = e.getValue();
    Obj.updateLine();
    }

public HLTScroll(HighlightText t)
    {
    Obj = t;
    }

}

}
