

package viror.teach.anim.net;

import viror.teach.anim.*;
import java.awt.*;

/** An internal link can be used to either connect two SoftwareComponents or
 *  a SoftwareComponent to a Router. It does not have all the fancyness of
 *  a link, rather it is symbolic for computer-internal communication.
 *  InternalLinks can only be connected to Routers running any type of
 *  PackageHandler. */

public class InternalLink extends AbstractLink
{

public InternalLink()
  {
  super();
  Delay  = 50;
  addAllowedStartPoint("Router");
  addAllowedStartPoint("SoftwareComponent");
  addAllowedEndPoint("Router");
  addAllowedEndPoint("SoftwareComponent");
  setIdentification("InternalLink");
  }

/** sets the first component of the internal link. If the other component
 *  is already a "Router" this can only be a "SoftwareComponent". Otherwise both
 *  options are open. All others will throw an IllegalArgumentException.
 */
public void setStart(AnimStatic e)
  {
  if (getStart() != null && getStart().getIdentification().equals("Router"))
     addAllowedEndPoint("Router");
  if (e.getIdentification().equals("Router")) removeAllowedEndPoint("Router");
  super.setStart(e);
  }

/** Sets the secondary component of the internal link. The same rules
 *  as for setStart() apply. */
public void setEnd(AnimStatic e)
  {
  if (getEnd() != null && getEnd().getIdentification().equals("Router"))
     addAllowedEndPoint("Router");
  super.setEnd(e);
  if (e.getIdentification().equals("Router")) removeAllowedEndPoint("Router");
  }


/** This method must be called to initialize the path. It only works after the
 *  start and the end point have been set...
 */
public void generateRepresentation()
  {
  super.generateRepresentation();
  float mod = 5.0f;
  float[] dash = new float[4];
  dash[0] = 3.0f * mod; dash[1] = 5.0f * mod; dash[2] = 1.0f * mod ; dash[3] = 5.0f * mod;
  ((VectorRep) getPeer()).setStroke(new BasicStroke(0.5f, BasicStroke.CAP_SQUARE,
                                        BasicStroke.JOIN_BEVEL, 2.0f, dash, 0.0f));
  ((VectorRep) getPeer()).setPaint(Color.gray);
  }

public String toString()
  {
  return "InternalLink("+getStart()+","+getEnd()+")";
  }

}