
package viror.teach.anim.net;

import viror.teach.anim.*;
import java.util.*;
import java.net.*;
import gitzel.util.*;
import javax.swing.ImageIcon;
import java.awt.Image;

/** This class is basically a collection of constants that can be
  set and gotten via static methods. There is no public constructor
  rather we use a factory method which gives us a reference on the
  single instance of this class or creates it if the method has
  never been called before.
  Another function is to associate pictures with names - this can be used
  by classes who want to send packages and need some bitmap for them.
  The network semantics can be loaded from an URL. */
public class NetworkSemantics
{

/** The single and only instance of this class. */
protected static NetworkSemantics Semantics;

  ///////////////

/** Here all images are stored in association with a name */
private Map IconList;

/** Here are all scales that are <i>recommended</i> for the given image. Note
 *  that this value does not have to be used at all. */
private Map ScaleList;

/** This Map stores the strings which describe the different
  package types */
Map PType;

/** The maximum number of package types. */
int Types;

  ///////////////

/** The constructor is protected and is called by getSemantics */
protected NetworkSemantics()
    {
    PType = new HashMap();
    setNumberOfTypes(10);
    IconList = new HashMap();
    ScaleList = new HashMap();
    }

/** This static method returns the current instance */
public static NetworkSemantics getSemantics()
    {
    if (Semantics == null) Semantics = new NetworkSemantics();
    return Semantics;
    }


  /////////////////


/** Gets the text for this package type */
public String getPackageType(int type)
    {
    String text = null;
    if (type > Types || type < 0)
      throw new ArrayIndexOutOfBoundsException();
      text = (String) PType.get(new Integer(type));
    if (text == null) text = "Type" + type;
    return text;
    }

/** sets a text for a certain package type.
 @param type The internal int value describing the package
 @param text The name of the package that is to be displayed
          towards the user. */
public void setPackageType(int type, String text)
  {
  PType.put(new Integer(type), text);
  if (type >= Types) Types = type;
  }

/** Returns the highest package type int value that exists within
  this context. */
public int getNumberOfTypes()
    {
    return Types;
    }

public void setNumberOfTypes(int i)
    {
    Types = i;
    }

/** This adds a new picture to the List of available graphics, stored under
 *  'name' */
public void addImage(String name, Image im)
  {
  IconList.put(name, im);
  }

/** Returns the image stored under the key 'name'. Usually a scaling factor is
 *  recommended for a picture (see getScale() for details. */
public Image getImage(String name)
  {
  return (Image) IconList.get(name);
  }

/** This adds a new scaling recommendation to the list of available scales. */
public void addScale(String name, double scale)
  {
  ScaleList.put(name, new Double(scale));
  }

/** Returns a scaling value that was recommended for the image name (the image
 *  can be retrieved via getImage(name)). The default recommendation is 1.0. */
public double getScale(String name)
  {
  double ret = 1.0;
  try
    {
    ret = ((Double) ScaleList.get(name)).doubleValue();
    } catch (Exception e) {}
  return ret;
  }

/** Reads an ini file from the given url. The syntax of the file is the
 *  following:
 *  <pre>           TYPES
 *                  packagetypenumber name
 *                  .....
 *                  IMAGES
 *                  name image_url scale_factor
 *                  .....</pre>
 * Note that the images will be read relatively to the context given by the applet.
 * Note that applet can be null if no images are to be loaded.
 **/
public void initFromURL(URL url, java.applet.Applet app) throws java.io.IOException,
                                                              MalformedURLException
  {
  URL context = app.getCodeBase();
  BasicScriptReader red = new BasicScriptReader(url.openStream());
  int state = 0;
  String[] dat;
  int type = 0;
  Iterator it = red.iterator();
  URL imurl = null;
  double scale = 1.0;
  java.awt.MediaTracker m = new java.awt.MediaTracker(app);
  while(it.hasNext())
    {
    dat = (String[]) it.next();
    switch (state)
      {
      case 0:
       if (dat.length == 1 && dat[0].equals("IMAGES"))
          {
          // no types will be defined
          state = 2;
          break;
          }
       if (dat.length != 1 || !dat[0].equals("TYPES"))
          {
          System.err.println(url + " has syntax error; should start with TYPES!");
          return;
          }
       state = 1;
      break;
      case 1:
       if (dat.length != 2)
         {
         System.err.println(url + " has syntax error in type definitions!");
         return;
         }
       try {
         type = new Integer(dat[0]).intValue();
       } catch (Exception e)
            {
            System.err.println(url+ " syntax error:" + dat[0] + " is not an int!");
            return;
            }
       setPackageType(type, dat[1]);
      break;
      case 2:
       if (dat.length != 3)
         {
         System.err.println(url + " has fouled syntax in the IMAGE part.");
         return;
         }
       try {
         imurl = new URL(context, dat[1]);
         scale = new Double(dat[2]).doubleValue();
         } catch (MalformedURLException e)
             {
             System.err.println(url + " contains a bad url: " + dat[1]);
             return;
             }
           catch (NumberFormatException e)
             {
             System.err.println(url + " is fouled: "+dat[2]+" is not a number!");
             scale = 1.0;
             }
       addImage(dat[0], retrieveImage(new URL(context, dat[1]), app, m));
       addScale(dat[0], scale);
      break;
      }
    }
  try { m.waitForAll(); } catch (Exception e) {}
  }


/** This method gets an image from the given URL. */
private Image retrieveImage(URL url, java.applet.Applet app, java.awt.MediaTracker m)
  {
  Image tmp = app.getImage(url);
  m.addImage(tmp, 0);
  return tmp;
  }
}
