

package viror.teach.anim.net;

import viror.teach.anim.UndoCustomizer;
import java.beans.PropertyChangeListener;
import javax.swing.*;
import java.awt.*;
import java.beans.PropertyChangeEvent;
import gitzel.util.TableGUI;

/**
 * This is no real customizer insofar as it does not have an editable mode
 * (i.e. new methods can be specified). The view mode allows editing of the
 * currently existing properties however...
 * @author Ralf Gitzel
 * @version 1.0
 */
public class OOComponentCustomizer extends JPanel implements UndoCustomizer,
                                                      PropertyChangeListener
{

/** This panel holds all buttons with methods this object can perform */
private JPanel Buttons;

/** This panel holds all the Properties of this object */
private JPanel Fields;

/** The array of buttons */
private JButton[] MethodCall;

/** The array of text fields */
private JTextField[] PropertyDisplay;

/** The array of labels associated with the text fields */
private JLabel[] PropertyLabel;

/** The object represented by this customizer */
private OOComponent Data;

/** This GUITable is used for editing... */
TableGUI EditTable;

public OOComponentCustomizer(boolean edit)
  {
  if (!edit) initDisplay();
  else initEdit();
  }

public void undo()
  {
    //TODO: Implement this viror.teach.anim.UndoCustomizer method
  }

public void setObject(Object bean)
  {
  Data = (OOComponent) bean;

  }

public void addPropertyChangeListener(PropertyChangeListener listener)
  {
    //V ????
  }

public void removePropertyChangeListener(PropertyChangeListener listener)
  {
    //TODO: Implement this java.beans.Customizer method
  }

private void initDisplay()
  {
  setLayout(new GridLayout(2,1));
  Buttons = new JPanel();
  Fields  = new JPanel();
  add(Buttons);
  add(Fields);
  }

private void initEdit()
  {
  String[] names = new String[3];
  names[0] = "Name";
  names[1] = "Type";
  names[2] = "Initial Value";
  String[] choices = new String[2];
  choices[0] = "Method";
  choices[1] = "Property";
  TableGUI.TableEntry[] format = new TableGUI.TableEntry[3];
  format[0] = new TableGUI.StringEntry();
  format[1] = new TableGUI.ChoiceEntry(choices);
  format[2] = new TableGUI.StringEntry();
  EditTable = new TableGUI(names, format, 5);
  add(EditTable);
  }

public void propertyChange(PropertyChangeEvent evt)
  {
  Icon tempIcon = null;
  if (evt.getPropertyName().equals("MethodsEnabled"))
    {
    boolean b = ((Boolean) evt.getNewValue()).booleanValue();
    if (MethodCall == null) return;  // method call not initialized yet
    for(int i=0; i < MethodCall.length; i++)
      MethodCall[i].setEnabled(b);
    }
  if (evt.getPropertyName().equals("MethodTable"))
    {
    String[] meth = Data.getMethodNames();
    // remove old action listeners (dangling references)
    Component[] bs = Buttons.getComponents();
    if (bs != null) for(int i=0; i < bs.length; i++)
                     ((JButton) bs[i]).removeActionListener(Data);
    Buttons.removeAll();
    // set up the buttons
    if (meth.length != 0)
      {
      Buttons.setLayout(new GridLayout(meth.length,1));
      MethodCall = new JButton[meth.length];
      for (int i=0; i < meth.length; i++)
        {
        tempIcon = Data.getMethodIcon(meth[i]);
        if(tempIcon == null)
          MethodCall[i] = new JButton(meth[i]);
        else MethodCall[i] = new JButton(meth[i], tempIcon);
        MethodCall[i].setHorizontalAlignment(SwingConstants.LEFT);
        MethodCall[i].setHorizontalTextPosition(SwingConstants.TRAILING);
        MethodCall[i].setEnabled(Data.isMethodsEnabled());
        MethodCall[i].addActionListener(Data);
        Buttons.add(MethodCall[i]);
        }
      }
    }
  if (evt.getPropertyName().equals("Properties"))
    {
    String[] prop = Data.getPropertyNames();
    if (prop == null) return;  // not initialized yet!
    PropertyLabel   = new JLabel[prop.length];
    PropertyDisplay = new JTextField[prop.length];
    Fields.removeAll();
    Fields.setLayout(new GridLayout(prop.length, 2, 1, 1));
    for (int i = 0; i < prop.length; i++)
      {
      PropertyLabel[i]   = new JLabel(prop[i]);
      PropertyDisplay[i] = new JTextField(Data.getPropertyValue(prop[i]));
      PropertyDisplay[i].addActionListener(Data);
      Fields.add(PropertyLabel[i]);
      Fields.add(PropertyDisplay[i]);
      }
    }
  if (evt.getPropertyName().equals("TableEntry"))
    {
    // the TableGUI object has been updated
    String[] alt = (String[]) evt.getOldValue();
    String[] neu = (String[]) evt.getNewValue();
    if (alt != null)
      {
      // not a  new entry but changing of on old one or a deletion
      if (alt[1].equals("Method")) Data.removeMethod(alt[0]);
      else Data.removeProperty(alt[0]);
      }
    if (neu == null)
      {
      // not a deletion but a change or a new entry
      try {
      if (neu[1].equals("Method")) Data.addMethod(neu[0], new Package(0,3,Color.blue));
      else Data.addProperty(neu[0], neu[2]);
      } catch (Exception e) {}
      }

    }
  }

/** Returns the current value for a PropertyDisplay TextField with the key being
 *  the label text next to it. If the value does not exist, a null is returned. */
public String getPropertyField(String labelname)
  {
  if (PropertyLabel == null) return null;
  for(int i=0; i < PropertyLabel.length; i++)
    {
    if (PropertyLabel[i].equals(labelname)) return PropertyDisplay[i].getText();
    }
  return null;
  }

public void setEnabled(boolean en)
  {
  super.setEnabled(en);
  if (MethodCall != null) for(int i=0; i < MethodCall.length; i++)
    MethodCall[i].setEnabled(en);
  }
}