
package viror.teach.anim.net;

import viror.teach.anim.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.awt.*;
import javax.swing.*;
import java.beans.*;

/** This class allows the bean Package to be initialized - this is not
  done via properties for one simple reason; we also use this class
  to allow users to view this component when the animation is running.
  Therefore the other constructor. Specialized packages will probably
  have their own implementation of this. The payload cannot be edited
  this way - the idea is to generic so implementations that use
  payloads will have to derieve from this class. */
public class PackageCustomizer extends JPanel implements UndoCustomizer
{

  /** Is the GUI editable? */
  private boolean Editable;

  /** This array contains the names given for all types the
    package may have. This information is taken from the
    NetworkSemantics object. */
  String[] Type;

  /** The Title text of this customizer */
  String Titel = "Package of type: ";

  /** The title text for the payload section */
  String PayText = "Carried Data:";

  /** The package size text */
  String SizeText = "Package Size:";

  /** The Payload button text */
  private String AddPayText = "Add";

  /** The Payload button text */
  private String RemovePayText = "Remove";

  /** The Payload button text */
  private String EditPayText = "Change";

  /** The reference to the package */
  Package Data;

  /** The Package Type can be chosen from all available */
  JComboBox TypeChoice;

  /** The list where all payload elements are listed. */
  JList PayBox;

  /** The button to add a new payload line */
  JButton AddPay;

  /** The button to edit a payload entry */
  JButton EditPay;

  /** The button to remove payload entries */
  JButton RemovePay;

  /** The Customizer for the payload... */
  JComponent CustomPay;

  /** The text field for size */
  JTextField SizeField;

  /** add/remove listener */
  PropertyChangeSupport ListenerList;

  /** The backup values for undo... */
  private int USize;

  /** The backup values for undo... */
  private int UType;

  ///////////////////////////////////////


  /** The standard constructor used by bean tools. */
  public PackageCustomizer()
    {
    this(true);
    }

  /** If editable is true, the values shown in this JPanel can be
    altered */
  public PackageCustomizer(boolean editable)
    {
    Editable = editable;
    // first set up the text info from network semantics
    NetworkSemantics n = NetworkSemantics.getSemantics();
    int j = n.getNumberOfTypes();
    Type = new String[j];
    for(int i = 0; i < j; i++)
      {
      Type[i] = n.getPackageType(i);
      }

    // gui components
    setLayout(new BorderLayout());
    JPanel top = new JPanel();
    JPanel mid = new JPanel();      // payload
    JPanel midmid = new JPanel();   // list and customizer
    JPanel midbutt = new JPanel();  // payload buttons
    JPanel bot = new JPanel();

    PackageCListener pcl = new PackageCListener();
    TypeChoice = new JComboBox();
    TypeChoice.setEditable(false); // textfield part only
    for (int i=0; i < j; i++) TypeChoice.addItem(Type[i]);
    TypeChoice.setSelectedIndex(0);
    TypeChoice.addFocusListener(pcl);
    PayBox     = new JList();
    PayBox.addListSelectionListener(pcl);
    JScrollPane Scroll = new JScrollPane(PayBox);
    int temp = PayBox.getFont().getSize();
    Scroll.setPreferredSize(new Dimension(temp * 4, temp * 12));
    EditPay    = new JButton(EditPayText);
    EditPay.addActionListener(pcl);
    RemovePay  = new JButton(RemovePayText);
    RemovePay.addActionListener(pcl);
    AddPay     = new JButton(AddPayText);
    AddPay.addActionListener(pcl);
    SizeField  = new JTextField(4);
    SizeField.addFocusListener(pcl);

    top.add(new JLabel(Titel));
    top.add(TypeChoice);
    add(top, BorderLayout.NORTH);

    mid.setLayout(new BorderLayout());
    mid.add(new JLabel(PayText), BorderLayout.NORTH);
    midmid.setLayout(new BorderLayout());
    midmid.add(Scroll, BorderLayout.CENTER);
    midmid.add(CustomPay = BasicPayload.getCustomizer(Editable), BorderLayout.SOUTH);
    if (Editable)
      {
      midbutt.add(AddPay);
      midbutt.add(RemovePay);
      midbutt.add(EditPay);
      }
    mid.add(midmid, BorderLayout.CENTER);
    mid.add(midbutt, BorderLayout.SOUTH);
    add(mid, BorderLayout.CENTER);

    bot.add(new JLabel(SizeText));
    bot.add(SizeField);
    add(bot, BorderLayout.SOUTH);
    }

  /** Pass the object that this customizer will display/alter */
  public void setObject(Object data)
    {
    Data = (Package) data;
    SizeField.setText(""+Data.getPSize());
    SizeField.setEnabled(Editable);
    TypeChoice.setSelectedIndex(Data.getType());
    TypeChoice.setEnabled(Editable);
    PayBox.setListData(Data.getPayload());
    // Undo values
    UType = Data.getType();
    USize = Data.getPSize();
    }

  public void setPropertyChangeListener(PropertyChangeListener p)
    {
      ListenerList.addPropertyChangeListener(p);
    }

  public void removePropertyChangeListener(PropertyChangeListener p)
    {
      ListenerList.removePropertyChangeListener(p);
    }


  public void undo()
    {
    try
      {
      Data.setPSize(USize);
      Data.setType(UType);
      } catch(Exception e)
         {
         // This can never occur, since the values are backups...
         System.err.println("Major(!) error in " + this);
         }
    }

  public void setAddPayText(String newAddPayText)
  {
    AddPayText = newAddPayText;
  }

  public String getAddPayText()
  {
    return AddPayText;
  }

  public void setRemovePayText(String newRemovePayText)
  {
    RemovePayText = newRemovePayText;
  }

  public String getRemovePayText()
  {
    return RemovePayText;
  }

  public void setEditPayText(String newEditPayText)
  {
    EditPayText = newEditPayText;
  }

  public String getEditPayText()
  {
    return EditPayText;
  }







/** A small listener class to update the object's values according to the
 * entries given.
 */
class PackageCListener implements FocusListener, ActionListener, ListSelectionListener
  {

  public void focusGained(FocusEvent e)
    {
    }

  public void focusLost(FocusEvent e)
    {
    if (e.getSource() == TypeChoice)
      {
      int val = TypeChoice.getSelectedIndex();
      try
        {
        Data.setType(val);
        } catch (Exception ex)
          {
          TypeChoice.setSelectedIndex(UType);
          }
      }
    if (e.getSource() == SizeField)
      {
      try
        {
        Data.setPSize(new Integer(SizeField.getText()).intValue());
        } catch (Exception ex)
          {
          SizeField.setText("" + USize);
          }
      }
    }

  public void actionPerformed(ActionEvent e)
    {
    if (e.getSource() == AddPay)
      {
      BasicPayload p = ((PayloadCustomizer)CustomPay).spawnObject();
      try
        {
        Data.addPayload(p);
        } catch (Exception ex)
          {
          // should not occur, since we do not set it to null here
          return;
          }
      PayBox.setListData(Data.getPayload());
      }

    if (e.getSource() == RemovePay)
      {
        int del[] = PayBox.getSelectedIndices();
        for(int i = del.length-1; i >= 0; i--) Data.removePayload(del[i]);
        PayBox.removeSelectionInterval(del[0],del[del.length-1]);
        PayBox.setListData(Data.getPayload());
      }

    if (e.getSource() == EditPay)
      {
      int num[] = PayBox.getSelectedIndices();
      for (int i=0; i < num.length; i++)
        Data.setPayload(num[i], ((PayloadCustomizer) CustomPay).spawnObject());
      PayBox.setListData(Data.getPayload());
      }
    }

  public void valueChanged(ListSelectionEvent e)
    {
    int i = PayBox.getSelectedIndex();
    if (i == -1) return;
    ((Customizer) CustomPay).setObject(Data.getPayload(i));
    }

  }
}

