
package viror.teach.anim.net;

import viror.teach.anim.*;
import java.util.*;
import java.beans.PropertyVetoException;
import javax.swing.JComponent;
import java.lang.reflect.*;
import gitzel.util.BasicScriptReader;
import java.io.InputStream;
import java.net.URL;

/** The standard implementation for the package handler this is basically
    a glorified routing table with some additional frills. */
public class SimpleParseHandler implements PackageHandler, Progressive
{

/** This Map stores all the table entries in the 'routing table' */
private Map Data;

/** The router where this program is running. */
private Router Home;

/** The ProgressIndicators associated with this object */
private Set Indicators;

/** If true, ProgressIndicators will have their first indicator
  advanced by one for every package arrived. */
private boolean Progressive;

/** This class object must be changed by derieved classes in order to call
 *  the methods they have implemented. Just set it in the constructor the same
 *  way it is set here! */
Class thisClass = SimpleParseHandler.class;

  /////////////////////////

/* @param home The router where this 'program' is running */
public SimpleParseHandler(Router home)
    {
    Home = home;
    Data = new HashMap();
    Indicators = new HashSet();
    Progressive = true;
    }

  /////////////////////////

public void handlePackage(Package p)
    {
    SPHEntry toDo = (SPHEntry) Data.get(new Integer(p.getType()));
    if (toDo == null) return;
    // else: entry found
    if (!toDo.Method.equals(""))
      {
      Class[] param = new Class[1];
      param[0] = Package.class;
      Object[] value = new Object[1];
      value[0] = p;
      try {
	thisClass.getMethod(toDo.Method, param).invoke(this, value);
      } catch (NoSuchMethodException e)
	{
	System.err.println("Cannot call " + toDo.Method + ": " + e);
	}
        catch (Exception e)
	  {
	  System.err.println("Exception! "+e);
	  }
      }
    if (toDo.Outgoing != -1)
      {
      p.getPath().removeActor(p);
      Package out = (Package) p.clone();
      try {
	out.setType(toDo.Outgoing);
      } catch (PropertyVetoException e)
	{
	  System.err.println(this + ": " + toDo.Outgoing +
			     " is no legal package type!");
	}
      if (Progressive) fireProgress(1, true);
      Home.sendPackage(out, Home.getLink(toDo.Target));
      p.getList().removeElement(p);
      }
    }

public String toString()
    {
    return "SimpleParseHandler running on "+getHome();
    }

/** adds a new entry into the 'routing table'.
  @param index The type of the incoming package
  @param method The name of the method to be called. This only
         works for derieved classes - the method will be
	 searched via introspection and therefore this
	 method might throw a MethodNotFoundException. "" means that
	 no method should be called.
  @param out The type of the outgoing package. -1 means that no
         package will be sent. Note that the outgoing package will
	 be a clone of the incoming in all other respects.
  @param target The Router the info will be sent to. */
public void addEntry(int index, String method, int out, Router target)
    {
    SPHEntry t = new SPHEntry(method, out, target);
    Data.put(new Integer(index), t);
    }

/** removes an entry from the table */
public void removeEntry(int i)
    {
    Data.remove(new Integer(i));
    }

/** Returns the keys of the map as array */
public int[] getKeys()
    {
    Set s = Data.keySet();
    Iterator it = s.iterator();
    int size = s.size();
    int[] ret = new int[size];
    for(int i=0; i < size; i++)
      {
      ret[i] = ((Integer) it.next()).intValue();
      }
    return ret;
    }

/** Change a key value from oldKey to newKey. */
public void changeKey(int oldKey, int newKey)
  {
  SPHEntry e = (SPHEntry) Data.get(new Integer(oldKey));
  Data.remove(new Integer(oldKey));
  Data.put(new Integer(newKey), e);
  }

/** Returns all out package types sorted the same way as the result of
  getKeys() is. */
public int[] getOutTypes()
    {
    int[] key = getKeys();
    int[] ret = new int[key.length];
    for (int i=0; i < key.length; i++)
      {
      ret[i] = ((SPHEntry) Data.get(new Integer(key[i]))).Outgoing;
      }
    return ret;
    }

/** Returns the single out type associated with the key i */
public int getOutType(int i)
   {
   return ((SPHEntry) Data.get(new Integer(i))).Outgoing;
   }

/** Change the out type associated with key i */
public void setOutType(int i, int newOut)
  {
  ((SPHEntry) Data.get(new Integer(i))).Outgoing = newOut;
  }


/** Returns all called methods sorted the same way as the result of
  getKeys() is. */
public String[] getMethods()
    {
    int[] key = getKeys();
    String[] ret = new String[key.length];
    for (int i=0; i < key.length; i++)
      {
      ret[i] = ((SPHEntry) Data.get(new Integer(key[i]))).Method;
      }
    return ret;
    }

/** Returns the method saved under the key i */
public String getMethod(int i)
  {
  return ((SPHEntry) Data.get(new Integer(i))).Method;
  }

/** Change the method stored under key i */
public void setMethod(int i, String newM)
  {
  ((SPHEntry) Data.get(new Integer(i))).Method = newM;
  }


/** Returns all target routers sorted the same way as the result of
  getKeys() is. */
public Router[] getTargets()
    {
    int[] key = getKeys();
    Router[] ret = new Router[key.length];
    for (int i=0; i < key.length; i++)
      {
      ret[i] = ((SPHEntry) Data.get(new Integer(key[i]))).Target;
      }
    return ret;
    }

/** Returns the target for key i */
public Router getTarget(int i)
  {
  return ((SPHEntry) Data.get(new Integer(i))).Target;
  }

/** Sets the target for key i */
public void setTarget(int i, Router r)
  {
  ((SPHEntry) Data.get(new Integer(i))).Target = r;
  }

public void setHome(Router h)
    {
    Home = h;
    }

public Router getHome()
    {
    return Home;
    }


public JComponent getCustomizer(boolean edit)
    {
    return new SimpleParseHandlerCustomizer(edit);
    }

public void addProgressIndicator(ProgressIndicator i)
    {
    Indicators.add(i);
    }

public void removeProgressIndicator(ProgressIndicator i)
    {
    Indicators.remove(i);
    }

public void fireProgress(int stateAffected, boolean forward)
    {
    Iterator it = Indicators.iterator();
    ProgressIndicator ind = null;
    while (it.hasNext())
      {
      ind = (ProgressIndicator) it.next();
      if (forward) ind.advance(stateAffected);
      else ind.stepBack(stateAffected);
      }
    }

public void fireProgress(int stateAffected, int line)
  {
    Iterator it = Indicators.iterator();
    ProgressIndicator ind = null;
    while (it.hasNext())
      {
      ind = (ProgressIndicator) it.next();
      ind.setPosition(stateAffected, line);
      }
  }

/** Reads the routing table from an URL. The following syntax is required for
 *  the ini file:
 *  <p><pre>   IN  OUT METHOD TARGETROUTER</pre>
 *  In stands for the package type that will be handled this way, out for the
 *  type to be sent, method for the name of the method to be called and target
 *  router is the name of the router to be reached. Impossible lines will be
 *  ignored and the error stream will notify the user of this! A method name
 *  of null will result in no method to be called.
 *  IMPORTANT NOTE: The initialization should only occur after ALL links
 *  have been connected, otherwise the targets will not be considered valid.  */
public void initRoutingFromURL(URL url) throws java.io.IOException
  {
  InputStream ins = url.openStream();
  BasicScriptReader r = new BasicScriptReader(ins);
  int max = r.getLineNumber();
  int in = 0, out = 0;
  String method;
  Router routa = null;
  Iterator it;
  Link lnk;
  for (int i=0; i < max; i++)
    {
    String[] data = r.getEntry(i);
    if (data.length != 4)
      {
      System.err.println(this+ ": line "+i+" of ini-script is fouled and thus ignored!");
      break;
      }
    try
      {
      in  = new Integer(data[0]).intValue();
      out = new Integer(data[1]).intValue();
      } catch (Exception e)
        {
        System.err.println(this+ ": line "+i+" of ini-script is fouled - package type must be int!");
        break;
        }
     method = data[2];
     it = getHome().LinkList.iterator();
     while(it.hasNext())
       {
       lnk = (Link) it.next();
       if (lnk.getStart() == this.getHome()) routa = (Router) lnk.getEnd();
       else routa = (Router) lnk.getStart();
       if (routa.getName().equals(data[3])) break;
       }
     if (routa == null)
       {
       System.err.println(this+ ": line "+i+" of ini-script is fouled, no such target router!");
       break;
       }
     // now  that all is settled, add new entry to routing table
     if (method.equals("null")) this.addEntry(in, "", out, routa);
     else this.addEntry(in, method, out, routa);
     }
  }

  ///////// Methods to be called via introspection /////////

/** This method emits a sharp system beep when a package is sent.
 *  As you might know, all methods starting with the prefix table..
 *  will be listed as possible handler methods in the Customizer.
 *  To make them callable by the program, their signature should
 *  be parallel to this one (i.e. void return type and one Package
 *  as parameter.
 */
public void tableBeep(Package p)
    {
    java.awt.Toolkit.getDefaultToolkit().beep();
    }


  ///////// Inner Classes //////////


class SPHEntry
  {

  /** The method to be called when the package arrives */
  String Method;

  /** The package type to be sent. When this value is -1 no package will be
   *  sent!
   */
  int Outgoing;

  /** The target router. */
  Router Target;

  public SPHEntry(String m, int o, Router t)
    {
    Method = m; Outgoing = o; Target = t;
    }

  public String toString()
    {
    return "Entry: " + Method + " Type-" + Outgoing + " Target: " + Target;
    }
  }

}
