
package viror.teach.anim.net;

import viror.teach.anim.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;

public class SimpleParseHandlerCustomizer extends JPanel implements UndoCustomizer
{

  PropertyChangeSupport ListenerList;
  SimpleParseHandler Data;

  /** True, if the fields may be edited... */
  boolean Editable;

  /** Scroll the table information */
  JScrollBar Scroll;

  /** This button adds a new entry */
  JButton Add;

  /** The number of entries in the table that are visible on screen
      at the same time*/
  final static int Entries = 5;

  /** The offset - i.e. how many lines are in front of the top line */
  private int Offset = 0;

  /** The Panel where the data is displayed and can be altered... */
  JPanel Table;

  /** The undo values for all entries */
  private int UIn[];

  /** The undo values for all entries */
  private int UOut[];
  /** The undo values for all entries */
  private String UMethod[];

  /** The undo values for all entries */
  Router UTarget[];

  /** The key value before the current edit change. This value is needed
   *  to know which key to change, since the TextField now contains the new
   *  one and the old one would be lost otherwise. The difference between
   *  this value and UIn is, that UIn is the value that was there <i>in the
   *  beginning</i> and OldIn is the value before the <i>last change</i>.
   */
  private int[] OldIn;

  /** This field is used to store the values of a newly added entry */
  private int NewIn, NewOut;
  /** This field is used to store the values of a newly added entry */
  private String NewMethod;
  /** This field is used to store the values of a newly added entry */
  private Router NewTarget;

  /** This list stores all keys that have to be thrown out again in the case
   * of an undo (i.e. entries that are new).
   */
  private Set NewEntries;

  JTextField[] In;
  JTextField[] Out;
  JComboBox[] Method;
  JComboBox[] Target;

  /** The text for the add button... */
  private String AddText = "Add";

  /////////////

  SimpleParseHandlerCustomizer(boolean edit)
    {
    Editable = edit;
    NewEntries = new HashSet();
    ListenerList = new PropertyChangeSupport(this);
    setLayout(new BorderLayout());
    Table    = new JPanel(new GridLayout(Entries,4));
    Scroll   = new JScrollBar();
    In       = new JTextField[Entries];
    Out      = new JTextField[Entries];
    Method   = new JComboBox[Entries];
    Target   = new JComboBox[Entries];
    SPHCListener sphc = new SPHCListener();

    Add      = new JButton(AddText);
    Add.addActionListener(sphc);
    for(int i=0; i < Entries; i++)
      {
      In[i]       = new JTextField(10);
      In[i].addFocusListener(sphc);
      Out[i]      = new JTextField(10);
      Out[i].addFocusListener(sphc);
      Method[i]   = new JComboBox();
      Method[i].addFocusListener(sphc);
      Target[i]   = new JComboBox();
      Target[i].addFocusListener(sphc);
      Table.add(In[i]);
      Table.add(Out[i]);
      Table.add(Method[i]);
      Table.add(Target[i]);
      }

    JPanel buffer = new JPanel();
    buffer.add(Add);

    add(Table, BorderLayout.CENTER);
    add(Scroll, BorderLayout.EAST);
    if (Editable) add(buffer, BorderLayout.SOUTH);
    }

  /////////////

  protected void updateLines()
    {
    int[]    key    = Data.getKeys();
    int[]    out    = Data.getOutTypes();
    String[] method = Data.getMethods();
    Router[] target = Data.getTargets();
    /* In this for-loop we iterate through all fields and fill them
       with the data approriate for the position of the scrollbar.
       The last line will be empty for new entries, the rest (if visible
       due to very few lines) will be disabled. */
    for (int i=0; i < Entries; i++)
      {
      if (i+Offset < key.length)
	{
        if (Editable)
          {
          In[i].setBackground(SystemColor.window);
          Out[i].setBackground(SystemColor.window);
          Method[i].setBackground(SystemColor.window);
          Target[i].setBackground(SystemColor.window);
          }
        else
          {
          In[i].setBackground(SystemColor.text);
          Out[i].setBackground(SystemColor.text);
          Method[i].setBackground(SystemColor.text);
          Target[i].setBackground(SystemColor.text);
          }
        In[i].setEnabled(Editable);
        Out[i].setEnabled(Editable);
        Method[i].setEnabled(Editable);
        Target[i].setEnabled(Editable);
	In[i].setText(""+key[i+Offset]);
	if (out[i+Offset] != -1) Out[i].setText(""+out[i+Offset]);
        else Out[i].setText("");
        try
          {
	  Method[i].setSelectedItem(method[i+Offset].substring(5));
          } catch (Exception ex)
            {
            System.err.println(method[i+Offset] + " is wrong method name!!");
            }
	Target[i].setSelectedItem(target[i+Offset]);
	}
      else
	{
	if (i+Offset == key.length)
           {
           // the single field for new entries....
           if (Editable)
             {
             In[i].setBackground(SystemColor.textHighlight);
             Out[i].setBackground(SystemColor.textHighlight);
             Method[i].setBackground(SystemColor.textHighlight);
             Target[i].setBackground(SystemColor.textHighlight);
             }
           else
             {
             In[i].setBackground(SystemColor.text);
             Out[i].setBackground(SystemColor.text);
             Method[i].setBackground(SystemColor.text);
             Target[i].setBackground(SystemColor.text);
             }
           In[i].setEnabled(Editable);
           Out[i].setEnabled(Editable);
           Method[i].setEnabled(Editable);
           Target[i].setEnabled(Editable);

           In[i].setText("");
           Out[i].setText("");
           }
        else
          {
          // all following fields are to be disabled...
          In[i].setBackground(SystemColor.inactiveCaptionBorder);
          Out[i].setBackground(SystemColor.inactiveCaptionBorder);
          Method[i].setBackground(SystemColor.inactiveCaptionBorder);
          Target[i].setBackground(SystemColor.inactiveCaptionBorder);
          In[i].setEnabled(false);
          Out[i].setEnabled(false);
          Method[i].setEnabled(false);
          Target[i].setEnabled(false);
          }
	}
      }
    Scroll.setMaximum(key.length + 1);  // leave space for new entry
    }


  /** Pass the object that this customizer will display/alter */
  public void setObject(Object data)
    {
    Data = (SimpleParseHandler) data;

    UIn    = Data.getKeys();
    OldIn  = Data.getKeys();
    UOut    = Data.getOutTypes();
    UMethod = Data.getMethods();
    UTarget = Data.getTargets();

    // init the combobox Method
    String[] m = IntrospectionHelper.
                   getXMethods(SimpleParseHandler.class, "table");
    for (int j=0; j < Entries; j++)
      {
      Method[j].setEditable(false);
      Method[j].addItem(""); // if no method is to be called
      for(int i=0; i < m.length; i++)
	{
	Method[j].addItem(m[i]);
	}
      }

    // init combobox Target
    Router r = Data.getHome();
    int max = r.getNumLinks();
    for (int j=0; j < Entries; j++)
      {
      Target[j].setEditable(false);
      for(int i=0; i < max; i++)
	{
	Target[j].addItem(r.getGoal(i));
	}
      }

    updateLines();
    }

  public void addPropertyChangeListener(PropertyChangeListener p)
    {
      ListenerList.addPropertyChangeListener(p);
    }

  public void removePropertyChangeListener(PropertyChangeListener p)
    {
      ListenerList.removePropertyChangeListener(p);
    }

  public void setAddText(String newAddText)
  {
    AddText = newAddText;
    Add.setText(AddText);
  }

  public String getAddText()
  {
    return AddText;
  }


class SPHCListener implements FocusListener, AdjustmentListener, ActionListener
  {

  public void actionPerformed(ActionEvent e)
    {
    if (e.getSource() == Add)
      {
      Data.addEntry(NewIn, NewMethod, NewOut, NewTarget);
      NewEntries.add(new Integer(NewIn));
      updateLines();
      }
    }

  public void focusGained(FocusEvent e)
    {
    }

  public void focusLost(FocusEvent e)
    {
    int t = 0;
    for (int i=0; i < Entries; i++)
      if (e.getSource() == In[i])
        {
        // case one - a key is changed....
        t = i + Offset;
        if (t == Data.getKeys().length) // special case - a new entry!
          {
          try
            {
            NewIn = new Integer(In[i].getText()).intValue();
            } catch(Exception ex)
              {
              In[i].setText("");
              }
          return;
          }
        int newKey = 0;
        try
          {
          newKey = new Integer(In[i].getText()).intValue();
          } catch (Exception ex)
            {
            In[i].setText("" + UIn[t]);
            return;
            }
        Data.changeKey(OldIn[i], newKey);
        OldIn[i] = UIn[t];
        }
    for (int i=0; i < Entries; i++)
      if (e.getSource() == Out[i])
        {
        // case 2 - outgoing type has changed...
        t = i + Offset;
        if (t == Data.getKeys().length) // special case - a new entry!
          {
          try
            {
            NewOut = new Integer(Out[i].getText()).intValue();
            } catch(Exception ex)
              {
              Out[i].setText("");
              }
          return;
          }
        if (Out[i].getText().equals(""))
          {
          // special case... send no package!
          Data.setOutType(t, -1);
          return;
          }
        try
          {
          Data.setOutType(t, new Integer(Out[i].getText()).intValue());
          } catch (Exception ex)
            {
            Out[i].setText(""+UOut[t]);
            }
        }
      for (int i=0; i < Entries; i++)
        if (e.getSource() == Method[i])
          {
          // case 3 - method to call has changed...
          t = i + Offset;
          if (t == Data.getKeys().length) // special case - a new entry!
            {
            NewMethod = (String) Method[i].getSelectedItem();
            return;
            }
          Data.setMethod(t, "" + Method[i].getSelectedItem());
          }
      for (int i=0; i < Entries; i++)
        if (e.getSource() == Target[i])
          {
          // case 4 - target has to be changed...
          t = i + Offset;
          if (t == Data.getKeys().length) // special case - a new entry!
            {
            NewTarget = (Router) Target[i].getSelectedItem();
            return;
            }
          Data.setTarget(t, (Router) Target[i].getSelectedItem());
          }
    }

  public void adjustmentValueChanged(AdjustmentEvent e)
    {
    Offset = e.getValue();
    updateLines();
    }

  }

  public void undo()
  {
    /* We have to set all entries to their original values AND remove all
       new entries. Note that the new entries have to be removed before
       the value reset - otherwise indices would be wrong! */
    Iterator it = NewEntries.iterator();
    while (it.hasNext())
      {
      int i = ((Integer)it.next()).intValue();
      Data.removeEntry(i);
      }

    for (int j=0; j < UIn.length; j++)
      {
      Data.addEntry(UIn[j], UMethod[j], UOut[j], UTarget[j]);
      }
  }

}
